@@ -1,27 +1,35 @@
-/* logbench.cpp
- *
- * cxxtools - general purpose C++-toolbox
+/*
  * Copyright (C) 2003 Tommi Maekitalo
- *
+ * 
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
- *
+ * 
+ * As a special exception, you may use this file as part of a free
+ * software library without restriction. Specifically, if other files
+ * instantiate templates or use macros or inline functions from this
+ * file, or you compile this file and link it with other files to
+ * produce an executable, this file does not by itself cause the
+ * resulting executable to be covered by the GNU General Public
+ * License. This exception does not however invalidate any other
+ * reasons why the executable file might be covered by the GNU Library
+ * General Public License.
+ * 
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
- *
+ * 
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
- *
  */
 
 #include <cxxtools/log.h>
 #include <cxxtools/loginit.h>
 #include <cxxtools/smartptr.h>
+#include <cxxtools/refcounted.h>
 #include <iostream>
 #include <vector>
 #include <stdexcept>
@@ -30,14 +38,13 @@
 #include <sys/time.h>
 #include <iomanip>
 
-log_define("log")
-
 namespace bench
 {
   log_define("bench")
 
-  class Logtester : public cxxtools::AttachedThread
+  class Logtester : public cxxtools::RefCounted
   {
+      cxxtools::AttachedThread thread;
       unsigned long count;
       unsigned long loops;
       unsigned long enabled;
@@ -46,11 +53,18 @@ namespace bench
       Logtester(unsigned long count_,
                 unsigned long loops_,
                 unsigned long enabled_)
-        : count(count_),
+        : thread( cxxtools::callable(*this, &Logtester::run) ),
+          count(count_),
           loops(loops_),
           enabled(enabled_)
           { }
 
+      void start()
+      { thread.start(); }
+
+      void join()
+      { thread.join(); }
+
       void setCount(unsigned long count_)   { count = count_; }
       void setLoops(unsigned long loops_)   { loops = loops_; }
       void setEnabled(bool sw = true)       { enabled = sw; }
@@ -86,7 +100,7 @@ int main(int argc, char* argv[])
 
     log_init();
 
-    typedef std::vector<cxxtools::SmartPtr<bench::Logtester, cxxtools::ExternalRefCounted> > Threads;
+    typedef std::vector<cxxtools::SmartPtr<bench::Logtester> > Threads;
     Threads threads;
     for (unsigned t = 0; t < numthreads; ++t)
       threads.push_back(new bench::Logtester(count, loops.getValue() / numthreads.getValue(), enable));
@@ -100,6 +114,7 @@ int main(int argc, char* argv[])
 
       struct timeval tv0;
       struct timeval tv1;
+
       gettimeofday(&tv0, 0);
 
       if (threads.size() == 1)
@@ -109,7 +124,7 @@ int main(int argc, char* argv[])
       else
       {
         for (Threads::iterator it = threads.begin(); it != threads.end(); ++it)
-          (*it)->create();
+          (*it)->start();
         for (Threads::iterator it = threads.begin(); it != threads.end(); ++it)
           (*it)->join();
       }
