@@ -1,28 +1,35 @@
-/* netmsg.cpp
- *
- * cxxtools - general purpose C++-toolbox
+/*
  * Copyright (C) 2003 Tommi Maekitalo
- *
+ * 
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
  * License as published by the Free Software Foundation; either
  * version 2.1 of the License, or (at your option) any later version.
- *
+ * 
+ * As a special exception, you may use this file as part of a free
+ * software library without restriction. Specifically, if other files
+ * instantiate templates or use macros or inline functions from this
+ * file, or you compile this file and link it with other files to
+ * produce an executable, this file does not by itself cause the
+ * resulting executable to be covered by the GNU General Public
+ * License. This exception does not however invalidate any other
+ * reasons why the executable file might be covered by the GNU Library
+ * General Public License.
+ * 
  * This library is distributed in the hope that it will be useful,
  * but WITHOUT ANY WARRANTY; without even the implied warranty of
  * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
  * Lesser General Public License for more details.
- *
+ * 
  * You should have received a copy of the GNU Lesser General Public
  * License along with this library; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
- *
  */
 
-#include <cxxtools/udp.h>
-#include <cxxtools/dynbuffer.h>
+#include <cxxtools/net/udp.h>
 #include <cxxtools/arg.h>
 #include <cxxtools/loginit.h>
+#include <vector>
 #include <iostream>
 
 void usage(const char* progname)
@@ -31,7 +38,7 @@ void usage(const char* progname)
                "       " << progname << " -l [-h host] [-p port] [-s size] [-c] [-e] [-n]\n"
                "options:\n"
                "  -l             receiver-mode\n"
-               "  -h host        hostname (default 127.0.0.1 in sender, 0.0.0.0 in receiver)\n"
+               "  -h host        hostname (default localhost in sender, any in receiver)\n"
                "  -p port        udp-port to use\n"
                "  -s size        size of receive-buffer in bytes (default 1024)\n"
                "  -c             continuous-mode - don't stop after receiving message\n"
@@ -63,7 +70,7 @@ int main(int argc, char* argv[])
     if (receive)
     {
       cxxtools::Arg<unsigned> size(argc, argv, 's', 1024);
-      cxxtools::Arg<const char*> host(argc, argv, 'h', "0.0.0.0");
+      cxxtools::Arg<const char*> host(argc, argv, 'h');
       cxxtools::Arg<bool> continuous(argc, argv, 'c');
 
       if (argc > 1)
@@ -76,13 +83,13 @@ int main(int argc, char* argv[])
       if (timeout.isSet())
         receiver.setTimeout(timeout);
 
-      cxxtools::Dynbuffer<char> buffer(size);
+      std::vector<char> buffer(size);
 
       std::cout << "waiting for messages on port " << port << std::endl;
       do
       {
-        cxxtools::net::UdpReceiver::size_type s = receiver.recv(buffer.data(), size);
-        std::string msg(buffer.data(), s);
+        cxxtools::net::UdpReceiver::size_type s = receiver.recv(&buffer[0], size);
+        std::string msg(&buffer[0], s);
         std::cout << msg;
         if (!nonewline)
           std::cout << std::endl;
@@ -96,7 +103,7 @@ int main(int argc, char* argv[])
     }
     else
     {
-      cxxtools::Arg<const char*> host(argc, argv, 'h', "127.0.0.1");
+      cxxtools::Arg<const char*> host(argc, argv, 'h');
       cxxtools::Arg<bool> broadcast(argc, argv, 'b');
 
       if (argc <= 1)
