@@ -231,7 +231,7 @@ class QueryParams
       : unnamed_params(src.unnamed_params),
         named_params(src.named_params),
         parent(src.parent),
-        use_parent_values(use_parent_values)
+        use_parent_values(src.use_parent_values)
     { }
 
     /// returns the parent-object
@@ -294,7 +294,7 @@ class QueryParams
 
     /// get all unnamed parameters
     template <typename output_iterator>
-    void get(output_iterator& o) const
+    void get(output_iterator o) const
     { std::copy(unnamed_params.begin(), unnamed_params.end(), o); }
 
     /// add unnamed parameter to parent or this class if no parent
