@@ -33,7 +33,18 @@
 
 namespace cxxtools
 {
-  template <typename objectType>
+  /**
+      \param ObjectType The managed object type
+  */
+  template <typename ObjectType>
+  /** \brief Reference linking.
+
+      Reference linking means that no counter is required to keep track of
+      the smart pointer objects, but all smart pointers form a linked
+      list. When the list becomes empty the raw pointer si deleted. This
+      Model has the advantage that it does not need to allocate memory,
+      but is prone to circular dependencies.
+  */
   class RefLinked
   {
       mutable const RefLinked* prev;
@@ -45,49 +56,71 @@ namespace cxxtools
           next(0)
           { }
 
-      bool unlink(objectType* object)
+      //! \brief Unlink a smart pointer from a managed object
+      bool unlink(ObjectType* object)
       {
-        bool ret = false;
         if (object)
         {
           if (next == this)
           {
-            ret = true;
+            next = prev = 0;
+            return true;
           }
           else
           {
             next->prev = prev;
             prev->next = next;
+            next = prev = this;
           }
-          next = prev = this;
         }
-        return ret;
+        return false;
       }
 
-      void link(const RefLinked& ptr, objectType* object)
+      //! \brief Link a smart pointer to a managed object
+      void link(const RefLinked& ptr, ObjectType* object)
       {
         if (object)
         {
-          prev = &ptr;
-          next = ptr.next;
-          prev->next = this;
-          next->prev = this;
+          if (ptr.next)
+          {
+            prev = &ptr;
+            next = ptr.next;
+            prev->next = this;
+            next->prev = this;
+          }
+          else
+          {
+            prev = next = this;
+          }
         }
       }
+
   };
 
-  template <typename objectType>
+  /**
+      \param ObjectType The managed object type
+  */
+  template <typename ObjectType>
+  /** \brief Intrusive reference counting.
+
+      Intrusive reference couting means that the reference count is part of the
+      managed heap object. Linking and unlinking will only increase and decrease this
+      counter, but not delete it. The managed object needs to implement the functions
+      release() and addRef() and must delete itself if the counter reaches zero.
+  */
   class InternalRefCounted
   {
     protected:
-      bool unlink(objectType* object)
+      //! \brief unlink a smart pointer from a managed object
+      bool unlink(ObjectType* object)
       {
         if (object)
           object->release();
         return false;
       }
 
-      void link(const InternalRefCounted& ptr, objectType* object)
+      //! \brief link a smart pointer to a managed object
+      void link(const InternalRefCounted& ptr, ObjectType* object)
       {
         if (object)
           object->addRef();
@@ -95,7 +128,17 @@ namespace cxxtools
 
   };
 
-  template <typename objectType>
+  /**
+      \param ObjectType The managed object type
+  */
+  template <typename ObjectType>
+  /** \brief Non-intrusive reference counting.
+
+      Non-intrusive reference couting means that the reference count is not part of the
+      managed heap object but part of the policy. Linking and unlinking will increase and
+      decrease the policies counter and delete the managed object if it reaches zero. A
+      small amount of memory needs to be allocated for the counter variable.
+  */
   class ExternalRefCounted
   {
       unsigned* rc;
@@ -104,20 +147,21 @@ namespace cxxtools
       ExternalRefCounted()
         : rc(0)  { }
 
-      bool unlink(objectType* object)
+      //! \brief unlink a smart pointer from a managed object
+      bool unlink(ObjectType* object)
       {
         if (object && --*rc <= 0)
         {
           delete rc;
-          // no need to set rc to 0 since the pointer is either
-          // destroyed or another object is linked in
+          rc = 0;
           return true;
         }
         else
           return false;
       }
 
-      void link(const ExternalRefCounted& ptr, objectType* object)
+      //! \brief link a smart pointer to a managed object
+      void link(const ExternalRefCounted& ptr, ObjectType* object)
       {
         if (object)
         {
@@ -138,7 +182,7 @@ namespace cxxtools
         { return rc ? *rc : 0; }
   };
 
-  template <typename objectType>
+  template <typename ObjectType>
   class ExternalAtomicRefCounted
   {
       volatile atomic_t* rc;
@@ -147,20 +191,19 @@ namespace cxxtools
       ExternalAtomicRefCounted()
         : rc(0)  { }
 
-      bool unlink(objectType* object)
+      bool unlink(ObjectType* object)
       {
         if (object && atomicDecrement(*rc) <= 0)
         {
           delete rc;
-          // no need to set rc to 0 since the pointer is either
-          // destroyed or another object is linked in
+          rc = 0;
           return true;
         }
         else
           return false;
       }
 
-      void link(const ExternalAtomicRefCounted& ptr, objectType* object)
+      void link(const ExternalAtomicRefCounted& ptr, ObjectType* object)
       {
         if (object)
         {
@@ -178,30 +221,53 @@ namespace cxxtools
 
     public:
       atomic_t refs() const
-        { return rc ? *rc : 0; }
+        { return rc ? atomicGet(*rc) : 0; }
   };
 
-  template <typename objectType>
+  /**
+      \param ObjectType The managed object type
+  */
+  template <typename ObjectType>
+  /** \brief old name for DeletePolicy for compatibility.
+
+      The DefaultDestroyPolicy implements the method, which instructs the SmartPtr to free the
+      object which it helds by deleting it.
+  */
   class DefaultDestroyPolicy
   {
     public:
-      static void destroy(objectType* ptr)
+      static void destroy(ObjectType* ptr)
+      { delete ptr; }
+  };
+
+  /**
+      \param ObjectType The managed object type
+  */
+  template <typename ObjectType>
+  /** \brief deleter policy for smart pointer.
+
+      The DeletePolicy is actually an alternative name for DefaultDestroyPolicy.
+  */
+  class DeletePolicy
+  {
+    public:
+      static void destroy(ObjectType* ptr)
       { delete ptr; }
   };
 
   template <typename T>
   class FreeDestroyPolicy
   {
-    protected:
-      void destroy(T* ptr)
+    public:
+      static void destroy(T* ptr)
       { free(ptr); }
   };
 
-  template <typename objectType>
+  template <typename ObjectType>
   class ArrayDestroyPolicy
   {
     public:
-      static void destroy(objectType* ptr)
+      static void destroy(ObjectType* ptr)
       { delete[] ptr; }
   };
 
@@ -227,59 +293,59 @@ namespace cxxtools
    * makes it straight-forward to use.
    *
    */
-  template <typename objectType,
-            template <class> class ownershipPolicy = InternalRefCounted,
-            template <class> class destroyPolicy = DefaultDestroyPolicy>
-  class SmartPtr : public ownershipPolicy<objectType>,
-                   public destroyPolicy<objectType>
+  template <typename ObjectType,
+            template <class> class OwnershipPolicy = InternalRefCounted,
+            template <class> class DestroyPolicy = DefaultDestroyPolicy>
+  class SmartPtr : public OwnershipPolicy<ObjectType>,
+                   public DestroyPolicy<ObjectType>
   {
-      objectType* object;
-      typedef ownershipPolicy<objectType> ownershipPolicyType;
-      typedef destroyPolicy<objectType> destroyPolicyType;
+      ObjectType* object;
+      typedef OwnershipPolicy<ObjectType> OwnershipPolicyType;
+      typedef DestroyPolicy<ObjectType> DestroyPolicyType;
 
     public:
       SmartPtr()
         : object(0)
         {}
-      SmartPtr(objectType* ptr)
+      SmartPtr(ObjectType* ptr)
         : object(ptr)
-        { ownershipPolicyType::link(*this, ptr); }
+        { OwnershipPolicyType::link(*this, ptr); }
       SmartPtr(const SmartPtr& ptr)
         : object(ptr.object)
-        { ownershipPolicyType::link(ptr, ptr.object); }
+        { OwnershipPolicyType::link(ptr, ptr.object); }
       ~SmartPtr()
-        { if (ownershipPolicyType::unlink(object))
-            destroy(object); }
+        { if (OwnershipPolicyType::unlink(object))
+            DestroyPolicy<ObjectType>::destroy(object); }
 
       SmartPtr& operator= (const SmartPtr& ptr)
       {
         if (object != ptr.object)
         {
-          if (ownershipPolicyType::unlink(object))
-            destroy(object);
+          if (OwnershipPolicyType::unlink(object))
+            DestroyPolicy<ObjectType>::destroy(object);
 
           object = ptr.object;
 
-          ownershipPolicyType::link(ptr, object);
+          OwnershipPolicyType::link(ptr, object);
         }
         return *this;
       }
 
       /// The object can be dereferenced like the held object
-      objectType* operator->() const              { return object; }
+      ObjectType* operator->() const              { return object; }
       /// The object can be dereferenced like the held object
-      objectType& operator*() const               { return *object; }
+      ObjectType& operator*() const               { return *object; }
 
-      bool operator== (const objectType* p) const { return object == p; }
-      bool operator!= (const objectType* p) const { return object != p; }
-      bool operator< (const objectType* p) const  { return object < p; }
+      bool operator== (const ObjectType* p) const { return object == p; }
+      bool operator!= (const ObjectType* p) const { return object != p; }
+      bool operator< (const ObjectType* p) const  { return object < p; }
       bool operator! () const { return object == 0; }
       operator bool () const  { return object != 0; }
 
-      objectType* getPointer()              { return object; }
-      const objectType* getPointer() const  { return object; }
-      operator objectType* ()               { return object; }
-      operator const objectType* () const   { return object; }
+      ObjectType* getPointer()              { return object; }
+      const ObjectType* getPointer() const  { return object; }
+      operator ObjectType* ()               { return object; }
+      operator const ObjectType* () const   { return object; }
   };
 
 }
