@@ -1,5 +1,6 @@
 /*
  * Copyright (C) 2004-2007 Marc Boris Duerner
+ * Copyright (C) 2011 Tommi Maekitalo
  * 
  * This library is free software; you can redistribute it and/or
  * modify it under the terms of the GNU Lesser General Public
@@ -31,12 +32,11 @@
 
 #include <cxxtools/api.h>
 #include <cxxtools/char.h>
-#include <cxxtools/stringdata.h>
 
 #include <string>
 #include <iterator>
-#include <cassert>
 #include <stdexcept>
+#include <iosfwd>
 
 #include <cxxtools/config.h>
 
@@ -46,7 +46,7 @@ namespace std {
     @ingroup Unicode
 */
 template <>
-class basic_string< cxxtools::Char > {
+class CXXTOOLS_API basic_string< cxxtools::Char > {
     public:
         typedef cxxtools::Char value_type;
         typedef size_t size_type;
@@ -73,9 +73,7 @@ class basic_string< cxxtools::Char > {
         static const size_type npos = static_cast<size_type>(-1);
 
     public:
-        basic_string();
-
-        explicit basic_string( const allocator_type& a );
+        explicit basic_string( const allocator_type& a = allocator_type());
 
         basic_string(const cxxtools::Char* str, const allocator_type& a = allocator_type());
 
@@ -83,30 +81,43 @@ class basic_string< cxxtools::Char > {
 
         basic_string(const wchar_t* str, size_type n, const allocator_type& a = allocator_type());
 
+        explicit basic_string(const std::string& str, const allocator_type& a = allocator_type());
+
+        explicit basic_string(const char* str, const allocator_type& a = allocator_type());
+
+        basic_string(const char* str, size_type n, const allocator_type& a = allocator_type());
+
         basic_string(const cxxtools::Char* str, size_type n, const allocator_type& a = allocator_type());
 
-        basic_string(size_type n, cxxtools::Char c);
+        basic_string(size_type n, cxxtools::Char c, const allocator_type& a = allocator_type());
 
         basic_string(const basic_string& str);
 
-        basic_string(const basic_string& str, size_type pos);
+        basic_string(const basic_string& str, const allocator_type& a);
 
-        basic_string(const basic_string& str, size_type pos, size_type n);
+        basic_string(const basic_string& str, size_type pos, const allocator_type& a = allocator_type());
 
-        basic_string(const basic_string& str, size_type pos, size_type n, const allocator_type& a);
+        basic_string(const basic_string& str, size_type pos, size_type n, const allocator_type& a = allocator_type());
 
-        basic_string(const cxxtools::Char* begin, const cxxtools::Char* end);
+        basic_string(const cxxtools::Char* begin, const cxxtools::Char* end, const allocator_type& a = allocator_type());
+
+        template <typename InputIterator>
+        basic_string(InputIterator begin, InputIterator end, const allocator_type& a = allocator_type());
 
         ~basic_string();
 
     public:
-        iterator begin();
+        iterator begin()
+        { return privdata_rw(); }
 
-        iterator end();
+        iterator end()
+        { return privdata_rw() + length(); }
 
-        const_iterator begin() const;
+        const_iterator begin() const
+        { return privdata_ro(); }
 
-        const_iterator end() const;
+        const_iterator end() const
+        { return privdata_ro() + length(); }
 
 #ifdef HAVE_STRING_REVERSE_ITERATOR
         reverse_iterator rbegin()
@@ -123,47 +134,29 @@ class basic_string< cxxtools::Char > {
 #endif
 
         reference operator[](size_type n)
-        {
-            this->detach( _data->length() );
-            _data->setBusy();
-            return *(_data->str() + n);
-        }
+        { return privdata_rw()[n]; }
 
         const_reference operator[](size_type n) const
-        { return *(_data->str() + n); }
+        { return privdata_ro()[n]; }
 
         reference at(size_type n)
-        {
-            if( n > this->size() ) {
-                throw std::out_of_range("The given at-value is out of range");
-            }
-            this->detach( _data->length() );
-            _data->setBusy();
-            return *(_data->str() + n);
-        }
+        { return privdata_rw()[n]; }
 
         const_reference at(size_type n) const
-        {
-            if( n > this->size() ) {
-                throw std::out_of_range("The given at-value is out of range");
-            }
-            return *(_data->str() + n);
-        }
+        { return privdata_ro()[n]; }
 
     public:
         void push_back(cxxtools::Char ch)
         { this->append(1, ch); }
 
-        // untested
         void resize( size_t n, cxxtools::Char ch = value_type() );
 
-        // untested
         void reserve(size_t n = 0);
 
         void swap(basic_string& str);
 
         allocator_type get_allocator() const
-        { return _data->get_allocator(); }
+        { return _data; }
 
         size_type copy(cxxtools::Char* a, size_type n, size_type pos = 0) const;
 
@@ -174,31 +167,52 @@ class basic_string< cxxtools::Char > {
         { return basic_string(*this, pos); }
 
     public:
-        size_type length() const;
+        size_type length() const
+        { return isShortString() ? shortStringLength() : longStringLength(); }
 
-        size_type size() const;
+        size_type size() const
+        { return length(); }
 
-        bool empty() const;
+        bool empty() const
+        { return length() == 0; }
 
-        size_type max_size() const;
+        size_type max_size() const
+        { return ( size_type(-1) / sizeof(cxxtools::Char) ) - 1; }
 
-        size_type capacity() const;
+        size_type capacity() const
+        { return isShortString() ? shortStringCapacity() : longStringCapacity(); }
 
         const cxxtools::Char* data() const
-        { return this->c_str(); }
+        { return privdata_ro(); }
 
-        const cxxtools::Char* c_str() const;
+        const cxxtools::Char* c_str() const
+        { return privdata_ro(); }
 
         basic_string& assign(const basic_string& str);
 
         basic_string& assign(const basic_string& str, size_type pos, size_type n);
 
+        basic_string& assign(const string& str);
+
+        basic_string& assign(const string& str, size_type pos, size_type n);
+
+        basic_string& assign(const wchar_t* str);
+
+        basic_string& assign(const wchar_t* str, size_type n);
+
         basic_string& assign(const cxxtools::Char* str);
 
         basic_string& assign(const cxxtools::Char* str, size_type length);
 
+        basic_string& assign(const char* str);
+
+        basic_string& assign(const char* str, size_type length);
+
         basic_string& assign(size_type n, cxxtools::Char c);
 
+        template <typename InputIterator>
+        basic_string& assign(InputIterator begin, InputIterator end);
+
         basic_string& append(const cxxtools::Char* str);
 
         basic_string& append(const cxxtools::Char* str, size_type n);
@@ -209,6 +223,9 @@ class basic_string< cxxtools::Char > {
 
         basic_string& append(const basic_string& str, size_type pos, size_type n);
 
+        template <typename InputIterator>
+        basic_string& append(InputIterator begin, InputIterator end);
+
         basic_string& append(const cxxtools::Char* begin, const cxxtools::Char* end);
 
         basic_string& insert(size_type pos, const cxxtools::Char* str);
@@ -229,7 +246,8 @@ class basic_string< cxxtools::Char > {
         //template <typename InputIterator>
         //basic_string& insert(iterator p, InputIterator first, InputIterator last);
 
-        void clear();
+        void clear()
+        { setLength(0); }
 
         basic_string& erase(size_type pos = 0, size_type n = npos);
 
@@ -262,8 +280,19 @@ class basic_string< cxxtools::Char > {
 
         int compare(const cxxtools::Char* str) const;
 
+        int compare(const cxxtools::Char* str, size_type n) const;
+
         int compare(const wchar_t* str) const;
 
+        int compare(const wchar_t* str, size_type n) const;
+
+        int compare(const std::string& str) const
+        { return compare(str.data(), str.length()); }
+
+        int compare(const char* str) const;
+
+        int compare(const char* str, size_type n) const;
+
         int compare(size_type pos, size_type n, const basic_string& str) const;
 
         int compare(size_type pos, size_type n, const basic_string& str, size_type pos2, size_type n2) const;
@@ -338,9 +367,9 @@ class basic_string< cxxtools::Char > {
         size_type find_last_not_of(cxxtools::Char ch, size_type pos = npos) const;
 
     public:
-        void detach(size_type reserveSize);
+        std::string narrow(char dfault = '?') const;
 
-        std::string narrow() const;
+        static basic_string widen(const char* str);
 
         static basic_string widen(const std::string& str);
 
@@ -354,6 +383,12 @@ class basic_string< cxxtools::Char > {
         basic_string& operator=(const basic_string& str)
         { return this->assign(str); }
 
+        basic_string& operator=(const string& str)
+        { return this->assign(str); }
+
+        basic_string& operator=(const char* str)
+        { return this->assign(str); }
+
         basic_string& operator=(const cxxtools::Char* str)
         { return this->assign(str); }
 
@@ -369,13 +404,79 @@ class basic_string< cxxtools::Char > {
         basic_string& operator+=(cxxtools::Char c)
         { return this->append(1, c); }
 
-        const cxxtools::StringData& sdata() const
+    private:
+        struct Ptr
         {
-            return *_data;
-        }
+            cxxtools::Char* _begin;
+            cxxtools::Char* _end;
+            cxxtools::Char* _capacity;
+        };
+
+        static const unsigned _minN = (sizeof(Ptr) / sizeof(uint32_t)) + 1;
+        static const unsigned _shortStringSize = _minN < 8 ? 8 : _minN;
+
+        struct Data : public allocator_type
+        {
+            Data(const allocator_type& a)
+            : allocator_type(a)
+            {
+                u.shortdata[0] = 0;
+                u.shortdata[_shortStringSize - 1] = _shortStringSize - 1;
+            }
+
+            union
+            {
+                Ptr ptr;
+                uint32_t shortdata[_shortStringSize];
+            } u;
+
+        } _data;
 
     private:
-        cxxtools::StringData* _data;
+        const cxxtools::Char* privdata_ro() const
+        { return isShortString() ? shortStringData() : longStringData(); }
+        cxxtools::Char* privdata_rw()
+        { return isShortString() ? shortStringData() : longStringData(); }
+
+        void privreserve(size_t n);
+
+        bool isShortString() const                    { return shortStringMagic() != 0xffff; }
+        void markLongString()                         { shortStringMagic() = 0xffff; }
+        const cxxtools::Char* shortStringData() const { return reinterpret_cast<const cxxtools::Char*>(&_data.u.shortdata[0]); }
+        cxxtools::Char* shortStringData()             { return reinterpret_cast<cxxtools::Char*>(&_data.u.shortdata[0]); }
+        uint32_t  shortStringMagic() const            { return _data.u.shortdata[_shortStringSize - 1]; }
+        uint32_t& shortStringMagic()                  { return _data.u.shortdata[_shortStringSize - 1]; }
+        size_type shortStringLength() const           { return _shortStringSize - 1 - shortStringMagic(); }
+        size_type shortStringCapacity() const         { return _shortStringSize - 1; }
+        void setShortStringLength(size_type n)        { shortStringData()[n] = cxxtools::Char(0); shortStringMagic() = _shortStringSize - n - 1; }
+        void shortStringAssign(const cxxtools::Char* str, size_type n)
+        {
+            traits_type::copy(shortStringData(), str, n);
+            shortStringData()[n] = cxxtools::Char(0);
+            shortStringMagic() = _shortStringSize - n - 1;
+        }
+        void shortStringAssign(const wchar_t* str, size_type n)
+        {
+            for (size_type nn = 0; nn < n; ++nn)
+                shortStringData()[nn] = str[nn];
+            shortStringData()[n] = cxxtools::Char(0);
+            shortStringMagic() = _shortStringSize - n - 1;
+        }
+
+        const cxxtools::Char* longStringData() const    { return _data.u.ptr._begin; }
+        cxxtools::Char* longStringData()                { return _data.u.ptr._begin; }
+        size_type longStringLength() const              { return _data.u.ptr._end - _data.u.ptr._begin; }
+        size_type longStringCapacity() const            { return _data.u.ptr._capacity - _data.u.ptr._begin; }
+        void setLength(size_type n)
+        {
+            if (isShortString())
+                setShortStringLength(n);
+            else
+            {
+                _data.u.ptr._end = _data.u.ptr._begin + n;
+                _data.u.ptr._begin[n] = cxxtools::Char::null();
+            }
+        }
     };
 
     inline basic_string<cxxtools::Char> operator+(const basic_string<cxxtools::Char>& a, const basic_string<cxxtools::Char>& b)
@@ -406,6 +507,21 @@ class basic_string< cxxtools::Char > {
     inline bool operator==(const basic_string<cxxtools::Char>& a, const wchar_t* b)
     { return a.compare(b) == 0; }
 
+    inline bool operator==(const wchar_t* b, const basic_string<cxxtools::Char>& a)
+    { return a.compare(b) == 0; }
+
+    inline bool operator==(const basic_string<cxxtools::Char>& a, const char* b)
+    { return a.compare(b) == 0; }
+
+    inline bool operator==(const char* b, const basic_string<cxxtools::Char>& a)
+    { return a.compare(b) == 0; }
+
+    inline bool operator==(const basic_string<cxxtools::Char>& a, const std::string& b)
+    { return a.compare(b) == 0; }
+
+    inline bool operator==(const std::string& b, const basic_string<cxxtools::Char>& a)
+    { return a.compare(b) == 0; }
+
     // operator !=
     inline bool operator!=(const basic_string<cxxtools::Char>& a, const basic_string<cxxtools::Char>& b)
     { return a.compare(b) != 0; }
@@ -445,6 +561,8 @@ class basic_string< cxxtools::Char > {
     inline bool operator>(const basic_string<cxxtools::Char>& a, const wchar_t* b)
     { return a.compare(b) > 0; }
 
+    CXXTOOLS_API ostream& operator<< (ostream& out, const basic_string<cxxtools::Char>& str);
+
 } // namespace std
 
 
