@@ -28,11 +28,13 @@
 
 #include "clientimpl.h"
 #include <cxxtools/http/client.h>
+#include <cxxtools/net/uri.h>
 #include "parser.h"
 #include <cxxtools/ioerror.h>
 #include <cxxtools/textstream.h>
 #include <cxxtools/base64codec.h>
 #include <sstream>
+#include "config.h"
 
 #include <cxxtools/log.h>
 
@@ -87,6 +89,31 @@ ClientImpl::ClientImpl(Client* client, const net::AddrInfo& addrinfo)
     cxxtools::connect(_stream.buffer().inputReady, *this, &ClientImpl::onInput);
 }
 
+ClientImpl::ClientImpl(Client* client, const net::Uri& uri)
+: _client(client)
+, _parseEvent(_replyHeader)
+, _parser(_parseEvent, true)
+, _request(0)
+, _addrInfo(uri.host(), uri.port())
+, _stream(8192, true)
+, _chunkedIStream(_stream.rdbuf())
+, _username(uri.user())
+, _password(uri.password())
+, _contentLength(0)
+, _readHeader(true)
+, _chunkedEncoding(false)
+, _reconnectOnError(false)
+, _errorPending(false)
+{
+    if (uri.protocol() != "http")
+        throw std::runtime_error("only http is supported by http client");
+
+    _stream.attachDevice(_socket);
+    cxxtools::connect(_socket.connected, *this, &ClientImpl::onConnect);
+    cxxtools::connect(_stream.buffer().outputReady, *this, &ClientImpl::onOutput);
+    cxxtools::connect(_stream.buffer().inputReady, *this, &ClientImpl::onInput);
+}
+
 ClientImpl::ClientImpl(Client* client, SelectorBase& selector, const net::AddrInfo& addrinfo)
 : _client(client)
 , _parseEvent(_replyHeader)
@@ -109,6 +136,31 @@ ClientImpl::ClientImpl(Client* client, SelectorBase& selector, const net::AddrIn
 }
 
 
+ClientImpl::ClientImpl(Client* client, SelectorBase& selector, const net::Uri& uri)
+: _client(client)
+, _parseEvent(_replyHeader)
+, _parser(_parseEvent, true)
+, _request(0)
+, _addrInfo(uri.host(), uri.port())
+, _stream(8192, true)
+, _chunkedIStream(_stream.rdbuf())
+, _contentLength(0)
+, _readHeader(true)
+, _chunkedEncoding(false)
+, _reconnectOnError(false)
+, _errorPending(false)
+{
+    if (uri.protocol() != "http")
+        throw std::runtime_error("only http is supported by http client");
+
+    _stream.attachDevice(_socket);
+    cxxtools::connect(_socket.connected, *this, &ClientImpl::onConnect);
+    cxxtools::connect(_stream.buffer().outputReady, *this, &ClientImpl::onOutput);
+    cxxtools::connect(_stream.buffer().inputReady, *this, &ClientImpl::onInput);
+    setSelector(selector);
+}
+
+
 void ClientImpl::setSelector(SelectorBase& selector)
 {
     selector.add(_socket);
@@ -174,7 +226,7 @@ const ReplyHeader& ClientImpl::execute(const Request& request, std::size_t timeo
     }
 
     if (!_stream)
-        throw IOError( CXXTOOLS_ERROR_MSG("error sending HTTP request") );
+        throw IOError("error sending HTTP request");
 
     log_debug("read reply");
 
@@ -188,7 +240,7 @@ const ReplyHeader& ClientImpl::execute(const Request& request, std::size_t timeo
         reexecute(request);
 
         if (!_stream)
-            throw IOError( CXXTOOLS_ERROR_MSG("error sending HTTP request") );
+            throw IOError("error sending HTTP request");
 
         doparse();
     }
@@ -196,13 +248,13 @@ const ReplyHeader& ClientImpl::execute(const Request& request, std::size_t timeo
     log_debug("reply ready");
 
     if (_stream.fail())
-        throw IOError( CXXTOOLS_ERROR_MSG("failed to read HTTP reply") );
+        throw IOError("failed to read HTTP reply");
 
     if (_parser.fail())
-        throw IOError( CXXTOOLS_ERROR_MSG("invalid HTTP reply") );
+        throw IOError("invalid HTTP reply");
 
     if (!_parser.end())
-        throw IOError( CXXTOOLS_ERROR_MSG("incomplete HTTP reply header") );
+        throw IOError("incomplete HTTP reply header");
 
     return _replyHeader;
 }
@@ -225,19 +277,8 @@ void ClientImpl::readBody(std::string& s)
 
         log_debug("eod=" << _chunkedIStream.eod());
 
-        if (_chunkedIStream.eod())
-        {
-            _parser.readHeader();
-            doparse();
-
-            if (_parser.fail())
-                throw std::runtime_error("http parser failed"); // TODO define exception class
-        }
-        else
-            throw std::runtime_error("chunked stream not complete");
-
-        if (_chunkedIStream.fail())
-            throw IOError( CXXTOOLS_ERROR_MSG("error reading HTTP reply body") );
+        if (!_chunkedIStream.eod())
+            throw IOError("error reading HTTP reply body: incomplete chunked data stream");
     }
     else
     {
@@ -252,7 +293,7 @@ void ClientImpl::readBody(std::string& s)
             s += ch;
 
         if (_stream.fail())
-            throw IOError( CXXTOOLS_ERROR_MSG("error reading HTTP reply body") );
+            throw IOError("error reading HTTP reply body");
 
         //log_debug("body read: \"" << s << '"');
     }
@@ -279,6 +320,9 @@ std::string ClientImpl::get(const std::string& url, std::size_t timeout)
 
 void ClientImpl::beginExecute(const Request& request)
 {
+    if (_socket.selector() == 0)
+        throw std::logic_error("cannot run async http request without a selector");
+
     log_trace("beginExecute");
 
     _errorPending = false;
@@ -293,7 +337,7 @@ void ClientImpl::beginExecute(const Request& request)
             _stream.buffer().beginWrite();
             _reconnectOnError = true;
         }
-        catch (const cxxtools::IOError&)
+        catch (const IOError&)
         {
             log_debug("first write failed, so connection is not active any more");
 
@@ -337,6 +381,7 @@ void ClientImpl::sendRequest(const Request& request)
     static const char* date = "Date";
     static const char* host = "Host";
     static const char* authorization = "Authorization";
+    static const char* userAgent = "User-Agent";
 
     _stream << request.method() << ' '
             << request.url() << " HTTP/"
@@ -374,6 +419,11 @@ void ClientImpl::sendRequest(const Request& request)
         _stream << "\r\n";
     }
 
+    if (!request.header().hasHeader(userAgent))
+    {
+        _stream << "User-Agent: " PACKAGE_STRING " http client\r\n";
+    }
+
     if (!_username.empty() && !request.header().hasHeader(authorization))
     {
         std::ostringstream d;
@@ -406,7 +456,7 @@ void ClientImpl::onConnect(net::TcpSocket& socket)
         log_debug("request sent - begin write");
         _stream.buffer().beginWrite();
     }
-    catch (const std::exception& e)
+    catch (const std::exception& )
     {
         _errorPending = true;
         _client->replyFinished(*_client);
@@ -480,7 +530,7 @@ void ClientImpl::onInput(StreamBuffer& sb)
             sb.endRead();
 
             if (sb.device()->eof())
-                throw IOError( CXXTOOLS_ERROR_MSG("end of input") );
+                throw IOError("end of input");
 
             _reconnectOnError = false;
 
@@ -633,7 +683,14 @@ void ClientImpl::processBodyAvailable(StreamBuffer& sb)
             }
 
             if (_chunkedIStream.fail())
-                throw IOError( CXXTOOLS_ERROR_MSG("error reading HTTP reply body") );
+                throw IOError("error reading HTTP reply body");
+        }
+        else if( _chunkedIStream.eod() )
+        {
+            if( _replyHeader.hasHeader("Trailer") )
+                _parser.readHeader();
+            else
+                _client->replyFinished(*_client);
         }
 
         if (_socket.enabled())
@@ -653,14 +710,14 @@ void ClientImpl::processBodyAvailable(StreamBuffer& sb)
     {
         log_debug("content-length(pre)=" << _contentLength);
 
-        while (_stream.good() && _contentLength > 0 && sb.in_avail() > 0)                                            
+        while (_stream.good() && _contentLength > 0 && sb.in_avail() > 0)
         {
             _contentLength -= _client->bodyAvailable(*_client); // TODO: may throw exception
             log_debug("content-length(post)=" << _contentLength);
         }
 
         if (_stream.fail())
-            throw IOError( CXXTOOLS_ERROR_MSG("error reading HTTP reply body") );
+            throw IOError("error reading HTTP reply body");
 
         if( _contentLength <= 0 )
         {
@@ -690,6 +747,8 @@ void ClientImpl::cancel()
     _socket.close();
     _stream.clear();
     _stream.buffer().discard();
+
+    _chunkedIStream.reset();
 }
 
 
