@@ -34,6 +34,7 @@
 #include <cxxtools/streamcounter.h>
 #include <cxxtools/posix/pipestream.h>
 #include <cxxtools/systemerror.h>
+#include <cxxtools/arg.h>
 #include <list>
 #include <vector>
 #include <algorithm>
@@ -142,7 +143,7 @@ namespace cxxtools
     else if (loghost.isConnected())
       return udpmessage;
     else
-      return std::cout;
+      return std::cerr;
   }
 
   void LoggerImpl::logEnd(std::ostream& appender)
@@ -601,17 +602,20 @@ void log_init_cxxtools(cxxtools::Logger::log_level_type level)
   cxxtools::reinitializeLoggers();
 }
 
-void log_init_cxxtools(const std::string& propertyfilename)
+bool log_init_cxxtools(const std::string& propertyfilename)
 {
   cxxtools::Logger::setEnabled(false);
   cxxtools::getBaseLoggers().clear();
 
   std::ifstream in(propertyfilename.c_str());
-  log_init_cxxtools(in);
+  return log_init_cxxtools(in);
 }
 
-void log_init_cxxtools(std::istream& in)
+bool log_init_cxxtools(std::istream& in)
 {
+  if (!in)
+    return false;
+
   cxxtools::Logger::setEnabled(true);
 
   enum state_type {
@@ -646,9 +650,9 @@ void log_init_cxxtools(std::istream& in)
   std::string category;
   std::string filename;
   std::string host;
-  unsigned short int port;
-  unsigned fsize;
-  unsigned maxbackupindex;
+  unsigned short int port = 0;
+  unsigned fsize = 0;
+  unsigned maxbackupindex = 0;
   bool logprocess = false;
   std::string logprocessuser;
   std::string logprocessgroup;
@@ -948,24 +952,71 @@ void log_init_cxxtools(std::istream& in)
     }
   }
 
+  switch (state)
+  {
+    case state_filename:
+      cxxtools::LoggerImpl::setFile(filename);
+      break;
+
+    case state_port:
+      if (port > 0)
+        cxxtools::LoggerImpl::setLoghost(host, port);
+      break;
+
+    case state_fsize:
+      cxxtools::LoggerImpl::setMaxFileSize(fsize);
+      break;
+
+    case state_maxbackupindex:
+      cxxtools::LoggerImpl::setMaxBackupIndex(maxbackupindex);
+      break;
+
+    default:
+      ;
+  }
+
   if (logprocess)
     cxxtools::LoggerImpl::runLoggerProcess(logprocessuser, logprocessgroup);
 
   cxxtools::reinitializeLoggers();
+
+  return true;
 }
 
-void log_init_cxxtools()
+bool log_init_cxxtools(int argc, char* argv[])
+{
+  return log_init_cxxtools(std::string(argv[0]) + ".properties");
+}
+
+bool log_init_cxxtools(int argc, char* argv[], char optionchar)
+{
+  cxxtools::Arg<std::string> pfile(argc, argv, optionchar);
+  if (!pfile.isSet())
+    return false;
+
+  return log_init_cxxtools(pfile);
+}
+
+bool log_init_cxxtools(int argc, char* argv[], const char* option)
+{
+  cxxtools::Arg<std::string> pfile(argc, argv, option);
+  if (!pfile.isSet())
+    return false;
+
+  return log_init_cxxtools(pfile);
+}
+
+bool log_init_cxxtools()
 {
   char* LOGPROPERTIES = ::getenv("LOGPROPERTIES");
   if (LOGPROPERTIES)
-    log_init_cxxtools(LOGPROPERTIES);
-  else
-  {
-    struct stat s;
-    if (stat("log.properties", &s) == 0)
-      log_init_cxxtools("log.properties");
-    else
-      log_init_cxxtools(cxxtools::Logger::LOG_LEVEL_ERROR);
-  }
+    return log_init_cxxtools(LOGPROPERTIES);
+
+  struct stat s;
+  if (stat("log.properties", &s) == 0)
+    return log_init_cxxtools("log.properties");
+
+  log_init_cxxtools(cxxtools::Logger::LOG_LEVEL_ERROR);
+  return false;
 }
 
