@@ -27,8 +27,12 @@
  */
 
 #include "cxxtools/md5stream.h"
+#include "cxxtools/log.h"
+#include "md5.h"
 #include <cstring>
 
+log_define("cxxtools.md5stream")
+
 namespace cxxtools
 {
 
@@ -36,8 +40,15 @@ namespace cxxtools
 // Md5streambuf
 //
 Md5streambuf::Md5streambuf()
+  : context(new cxxtools_MD5_CTX())
 {
-  std::memset(digest, 0, 16);
+  log_debug("initialize MD5");
+  cxxtools_MD5Init(context);
+}
+
+Md5streambuf::~Md5streambuf()
+{
+  delete context;
 }
 
 std::streambuf::int_type Md5streambuf::overflow(
@@ -46,12 +57,14 @@ std::streambuf::int_type Md5streambuf::overflow(
   if (pptr() == 0)
   {
     // Ausgabepuffer ist leer - initialisieren
-    MD5Init(&context);
+    log_debug("initialize MD5");
+    cxxtools_MD5Init(context);
   }
   else
   {
     // konsumiere Zeichen aus dem Puffer
-    MD5Update(&context,
+    log_debug("process " << (pptr() - pbase()) << " bytes of data");
+    cxxtools_MD5Update(context,
               (const unsigned char*)pbase(),
               pptr() - pbase());
   }
@@ -81,7 +94,8 @@ int Md5streambuf::sync()
   if (pptr() != pbase())
   {
     // konsumiere Zeichen aus dem Puffer
-    MD5Update(&context, (const unsigned char*)pbase(), pptr() - pbase());
+    log_debug("process " << (pptr() - pbase()) << " bytes of data");
+    cxxtools_MD5Update(context, (const unsigned char*)pbase(), pptr() - pbase());
 
     // leere Ausgabepuffer
     setp(buffer, buffer + bufsize);
@@ -97,14 +111,21 @@ void Md5streambuf::getDigest(unsigned char digest_[16])
     if (pptr() != pbase())
     {
       // konsumiere Zeichen aus dem Puffer
-      MD5Update(&context, (const unsigned char*)pbase(), pptr() - pbase());
+      log_debug("process " << (pptr() - pbase()) << " bytes of data");
+      cxxtools_MD5Update(context, (const unsigned char*)pbase(), pptr() - pbase());
     }
 
     // deinitialisiere Ausgabepuffer
     setp(0, 0);
-
-    MD5Final(digest, &context);
   }
+  else
+  {
+    log_debug("initialize MD5");
+    cxxtools_MD5Init(context);
+  }
+
+  log_debug("finalize MD5");
+  cxxtools_MD5Final(digest, context);
 
   std::memcpy(digest_, digest, 16);
 }
@@ -125,6 +146,7 @@ const char* Md5stream::getHexDigest()
     *p++ = hex[md5[i] & 0xf];
   }
   *p = '\0';
+  log_debug("md5: " << hexdigest);
   return hexdigest;
 }
 
