@@ -35,7 +35,6 @@
 #include "cxxtools/xml/xmlerror.h"
 #include "cxxtools/textstream.h"
 #include "cxxtools/utf8codec.h"
-#include "cxxtools/sourceinfo.h"
 #include "cxxtools/log.h"
 #include <stdexcept>
 #include <iostream>
@@ -163,7 +162,7 @@ struct XmlReaderImpl
         virtual State* onAlpha(cxxtools::Char c, XmlReaderImpl& reader)
         {
             std::ostringstream msg;
-            msg << "unexpected alpha '" << c << '\'';
+            msg << "unexpected alpha '" << c.narrow() << '\'';
             syntaxError(msg.str().c_str(), reader.line());
             return this;
         }
@@ -294,7 +293,15 @@ struct XmlReaderImpl
         {
             if(c == ';')
             {
-                reader.resolveEntity(reader._token);
+                try
+                {
+                    reader.resolveEntity(reader._token);
+                }
+                catch (const std::exception&)
+                {
+                    throw XmlError("invalid entity " + reader._token.narrow(), reader.line());
+                }
+
                 reader._chars.content() += reader._token;
                 reader._token.clear();
                 return OnCharacters::instance();
@@ -312,6 +319,30 @@ struct XmlReaderImpl
     };
 
 
+    struct OnAttributeEntityReference : public State
+    {
+        virtual State* onAlpha(cxxtools::Char c, XmlReaderImpl& reader)
+        {
+            if(c == ';')
+            {
+                reader.resolveEntity(reader._token);
+                reader._attr.value() += reader._token;
+                reader._token.clear();
+                return OnAttributeValue::instance();
+            }
+
+            reader._token += c;
+            return this;
+        }
+
+        static State* instance()
+        {
+            static OnAttributeEntityReference _state;
+            return &_state;
+        }
+    };
+
+
     struct OnCharacters : public State
     {
         virtual State* onSpace(cxxtools::Char c, XmlReaderImpl& reader)
@@ -492,9 +523,63 @@ struct XmlReaderImpl
             return BeforeAttribute::instance();
         }
 
+        virtual State* onSpace(cxxtools::Char c, XmlReaderImpl& reader)
+        {
+            reader._attr.value() += c;
+            return this;
+        }
+
+        virtual State* onOpenBracket(cxxtools::Char c, XmlReaderImpl& reader)
+        {
+            reader._attr.value() += c;
+            return this;
+        }
+
+        virtual State* onCloseBracket(cxxtools::Char c, XmlReaderImpl& reader)
+        {
+            reader._attr.value() += c;
+            return this;
+        }
+
+        virtual State* onColon(cxxtools::Char c, XmlReaderImpl& reader)
+        {
+            reader._attr.value() += c;
+            return this;
+        }
+
+        virtual State* onSlash(cxxtools::Char c, XmlReaderImpl& reader)
+        {
+            reader._attr.value() += c;
+            return this;
+        }
+
+        virtual State* onEqual(cxxtools::Char c, XmlReaderImpl& reader)
+        {
+            reader._attr.value() += c;
+            return this;
+        }
+
+        virtual State* onExclam(cxxtools::Char c, XmlReaderImpl& reader)
+        {
+            reader._attr.value() += c;
+            return this;
+        }
+
+        virtual State* onQuest(cxxtools::Char c, XmlReaderImpl& reader)
+        {
+            reader._attr.value() += c;
+            return this;
+        }
+
         virtual State* onAlpha(cxxtools::Char c, XmlReaderImpl& reader)
         {
-            reader._attr.value() += c;;
+            if (c == '&')
+            {
+                reader._token.clear();
+                return OnAttributeEntityReference::instance();
+            }
+
+            reader._attr.value() += c;
             return this;
         }
 
@@ -564,6 +649,12 @@ struct XmlReaderImpl
             return this;
         }
 
+        virtual State* onColon(cxxtools::Char c, XmlReaderImpl& reader)
+        {
+            reader._attr.name() += c;
+            return this;
+        }
+
         static State* instance()
         {
             static OnAttributeName _state;
@@ -1174,7 +1265,7 @@ struct XmlReaderImpl
 
         virtual State* onAlpha(cxxtools::Char c, XmlReaderImpl& reader)
         {
-            reader._attr.value() += c;;
+            reader._attr.value() += c;
             return this;
         }
 
@@ -1463,7 +1554,14 @@ struct XmlReaderImpl
     { return _standalone; }
 
     EntityResolver& entityResolver()
-    { return _resolver; }
+    {
+       return _resolver;
+    }
+
+    const EntityResolver& entityResolver() const
+    {
+       return _resolver;
+    }
 
     size_t depth() const
     {
@@ -1526,7 +1624,7 @@ struct XmlReaderImpl
 
     void resolveEntity(String& str)
     {
-        str = _resolver.resolveEntity( str );
+        str = entityResolver().resolveEntity( str );
     }
 
     void appendContent(cxxtools::Char c)
@@ -1677,7 +1775,7 @@ const StartElement& XmlReader::nextElement()
         {
             case Node::EndDocument:
             {
-                throw std::logic_error("End of document" + CXXTOOLS_SOURCEINFO);
+                throw std::logic_error("End of document");
             }
             case Node::StartElement:
                 found = true;
@@ -1703,7 +1801,7 @@ const Node& XmlReader::nextTag()
         {
             case Node::EndDocument:
             {
-                throw std::logic_error("End of document" + CXXTOOLS_SOURCEINFO);
+                throw std::logic_error("End of document");
             }
             case Node::StartElement:
             case Node::EndElement:
