@@ -29,60 +29,63 @@
 #include "cxxtools/unit/registertest.h"
 #include "cxxtools/xmlrpc/service.h"
 #include "cxxtools/xmlrpc/httpclient.h"
-#include "cxxtools/xmlrpc/fault.h"
-#include "cxxtools/xmlrpc/remoteprocedure.h"
+#include "cxxtools/remoteexception.h"
+#include "cxxtools/remoteprocedure.h"
 #include "cxxtools/http/server.h"
 #include "cxxtools/eventloop.h"
 #include "cxxtools/log.h"
+#include <stdlib.h>
+#include <sstream>
 
 log_define("cxxtools.test.xmlrpc")
 
-
-struct Color
+namespace
 {
-    int red;
-    int green;
-    int blue;
-};
+    struct Color
+    {
+        int red;
+        int green;
+        int blue;
+    };
 
 
-typedef std::set<int> IntSet;
-typedef std::multiset<int> IntMultiset;
-typedef std::map<int, int> IntMap;
-typedef std::multimap<int, int> IntMultimap;
+    typedef std::set<int> IntSet;
+    typedef std::multiset<int> IntMultiset;
+    typedef std::map<int, int> IntMap;
+    typedef std::multimap<int, int> IntMultimap;
 
 
-void operator >>=(const cxxtools::SerializationInfo& si, Color& color)
-{
-    color.red = si.getValue<int>("red");
-    color.green = si.getValue<int>("green");
-    color.blue = si.getValue<int>("blue");
-}
+    void operator >>=(const cxxtools::SerializationInfo& si, Color& color)
+    {
+        si.getMember("red") >>= color.red;
+        si.getMember("green") >>= color.green;
+        si.getMember("blue") >>= color.blue;
+    }
 
 
-void operator <<=(cxxtools::SerializationInfo& si, const Color& color)
-{
-    si.addMember("red") <<= color.red;
-    si.addMember("green") <<= color.green;
-    si.addMember("blue") <<= color.blue;
+    void operator <<=(cxxtools::SerializationInfo& si, const Color& color)
+    {
+        si.addMember("red") <<= color.red;
+        si.addMember("green") <<= color.green;
+        si.addMember("blue") <<= color.blue;
+    }
+
 }
 
 
 class XmlRpcTest : public cxxtools::unit::TestSuite
 {
     private:
-        cxxtools::EventLoop* _loop;
+        cxxtools::EventLoop _loop;
         cxxtools::http::Server* _server;
         unsigned _count;
+        unsigned short _port;
 
     public:
         XmlRpcTest()
-        : cxxtools::unit::TestSuite("cxxtools-xmlrpc-Test")
+        : cxxtools::unit::TestSuite("xmlrpc"),
+          _port(8001)
         {
-            registerMethod("Fault", *this, &XmlRpcTest::Fault);
-            registerMethod("Exception", *this, &XmlRpcTest::Exception);
-            registerMethod("CallbackException", *this, &XmlRpcTest::CallbackException);
-            registerMethod("ConnectError", *this, &XmlRpcTest::ConnectError);
             registerMethod("Nothing", *this, &XmlRpcTest::Nothing);
             registerMethod("Boolean", *this, &XmlRpcTest::Boolean);
             registerMethod("Integer", *this, &XmlRpcTest::Integer);
@@ -96,6 +99,19 @@ class XmlRpcTest : public cxxtools::unit::TestSuite
             registerMethod("Multiset", *this, &XmlRpcTest::Multiset);
             registerMethod("Map", *this, &XmlRpcTest::Map);
             registerMethod("Multimap", *this, &XmlRpcTest::Multimap);
+            registerMethod("UnknownMethod", *this, &XmlRpcTest::UnknownMethod);
+            registerMethod("Fault", *this, &XmlRpcTest::Fault);
+            registerMethod("Exception", *this, &XmlRpcTest::Exception);
+            registerMethod("CallbackException", *this, &XmlRpcTest::CallbackException);
+            registerMethod("ConnectError", *this, &XmlRpcTest::ConnectError);
+            registerMethod("BigRequest", *this, &XmlRpcTest::BigRequest);
+
+            char* PORT = getenv("UTEST_PORT");
+            if (PORT)
+            {
+                std::istringstream s(PORT);
+                s >> _port;
+            }
         }
 
         void failTest()
@@ -105,98 +121,19 @@ class XmlRpcTest : public cxxtools::unit::TestSuite
 
         void setUp()
         {
-            _loop = new cxxtools::EventLoop();
-            _loop->setIdleTimeout(2000);
-            connect(_loop->timeout, *this, &XmlRpcTest::failTest);
-            connect(_loop->timeout, *_loop, &cxxtools::EventLoop::exit);
+            _loop.setIdleTimeout(2000);
+            connect(_loop.timeout, *this, &XmlRpcTest::failTest);
+            connect(_loop.timeout, _loop, &cxxtools::EventLoop::exit);
 
-            _server = new cxxtools::http::Server(*_loop, "127.0.0.1", 8001);
+            _server = new cxxtools::http::Server(_loop, "", _port);
+            _server->minThreads(1);
         }
 
         void tearDown()
         {
-            delete _loop;
             delete _server;
         }
 
-        ////////////////////////////////////////////////////////////
-        // Fault
-        //
-        void Fault()
-        {
-            cxxtools::xmlrpc::Service service;
-            service.registerMethod("multiply", *this, &XmlRpcTest::throwFault);
-            _server->addService("/calc", service);
-
-            cxxtools::xmlrpc::HttpClient client(*_loop, "127.0.0.1", 8001, "/calc");
-            cxxtools::xmlrpc::RemoteProcedure<bool> multiply(client, "multiply");
-            connect( multiply.finished, *this, &XmlRpcTest::onFault );
-            multiply.begin();
-
-            _loop->run();
-        }
-
-        void onFault(const cxxtools::xmlrpc::Result<bool>& result)
-        {
-            try
-            {
-                result.get();
-                CXXTOOLS_UNIT_ASSERT_MSG(false, "cxxtools::xmlrpc::Fault exception expected");
-            }
-            catch (const cxxtools::xmlrpc::Fault& e)
-            {
-                CXXTOOLS_UNIT_ASSERT_EQUALS(e.rc(), 7)
-                CXXTOOLS_UNIT_ASSERT_EQUALS(e.text(), "Fault")
-            }
-
-            _loop->exit();
-        }
-
-        bool throwFault()
-        {
-            throw cxxtools::xmlrpc::Fault("Fault", 7);
-            return false;
-        }
-
-        ////////////////////////////////////////////////////////////
-        // Exception
-        //
-        void Exception()
-        {
-            cxxtools::xmlrpc::Service service;
-            service.registerMethod("multiply", *this, &XmlRpcTest::throwException);
-            _server->addService("/calc", service);
-
-            cxxtools::xmlrpc::HttpClient client(*_loop, "127.0.0.1", 8001, "/calc");
-            cxxtools::xmlrpc::RemoteProcedure<bool> multiply(client, "multiply");
-            connect( multiply.finished, *this, &XmlRpcTest::onException );
-            multiply.begin();
-
-            _loop->run();
-        }
-
-        void onException(const cxxtools::xmlrpc::Result<bool>& result)
-        {
-            try
-            {
-                bool v = result.get();
-                CXXTOOLS_UNIT_ASSERT(false);
-            }
-            catch (const cxxtools::xmlrpc::Fault& e)
-            {
-                CXXTOOLS_UNIT_ASSERT_EQUALS(e.rc(), 0)
-                CXXTOOLS_UNIT_ASSERT_EQUALS(e.text(), "Exception")
-            }
-
-            _loop->exit();
-        }
-
-        bool throwException()
-        {
-            throw std::runtime_error("Exception");
-            return false;
-        }
-
         ////////////////////////////////////////////////////////////
         // Nothing
         //
@@ -204,22 +141,22 @@ class XmlRpcTest : public cxxtools::unit::TestSuite
         {
             cxxtools::xmlrpc::Service service;
             service.registerMethod("multiply", *this, &XmlRpcTest::multiplyNothing);
-            _server->addService("/calc", service);
+            _server->addService("/rpc", service);
 
-            cxxtools::xmlrpc::HttpClient client(*_loop, "127.0.0.1", 8001, "/calc");
-            cxxtools::xmlrpc::RemoteProcedure<bool> multiply(client, "multiply");
+            cxxtools::xmlrpc::HttpClient client(_loop, "", _port, "/rpc");
+            cxxtools::RemoteProcedure<bool> multiply(client, "multiply");
             connect( multiply.finished, *this, &XmlRpcTest::onNothingFinished );
 
             multiply.begin();
 
-            _loop->run();
+            _loop.run();
         }
 
-        void onNothingFinished(const cxxtools::xmlrpc::Result<bool>& r)
+        void onNothingFinished(const cxxtools::RemoteResult<bool>& r)
         {
-            CXXTOOLS_UNIT_ASSERT_EQUALS(r.get(), false)
+            CXXTOOLS_UNIT_ASSERT_EQUALS(r.get(), false);
 
-            _loop->exit();
+            _loop.exit();
         }
 
         bool multiplyNothing()
@@ -234,24 +171,24 @@ class XmlRpcTest : public cxxtools::unit::TestSuite
         {
             cxxtools::xmlrpc::Service service;
             service.registerMethod("multiply", *this, &XmlRpcTest::multiplyNothing);
-            _server->addService("/calc", service);
+            _server->addService("/rpc", service);
 
-            cxxtools::xmlrpc::HttpClient client(*_loop, "127.0.0.1", 8001, "/calc");
-            cxxtools::xmlrpc::RemoteProcedure<bool> multiply(client, "multiply");
+            cxxtools::xmlrpc::HttpClient client(_loop, "", _port, "/rpc");
+            cxxtools::RemoteProcedure<bool> multiply(client, "multiply");
             connect( multiply.finished, *this, &XmlRpcTest::onExceptionCallback );
 
             multiply.begin();
 
             _count = 0;
-            CXXTOOLS_UNIT_ASSERT_THROW(_loop->run(), std::runtime_error);
+            CXXTOOLS_UNIT_ASSERT_THROW(_loop.run(), std::runtime_error);
             CXXTOOLS_UNIT_ASSERT_EQUALS(_count, 1);
         }
 
-        void onExceptionCallback(const cxxtools::xmlrpc::Result<bool>& r)
+        void onExceptionCallback(const cxxtools::RemoteResult<bool>& r)
         {
             log_warn("exception callback");
             ++_count;
-            _loop->exit();
+            _loop.exit();
             throw std::runtime_error("my error");
         }
 
@@ -262,15 +199,15 @@ class XmlRpcTest : public cxxtools::unit::TestSuite
         {
             log_trace("ConnectError");
 
-            cxxtools::xmlrpc::HttpClient client(*_loop, "127.0.0.1", 8002, "/calc");
-            cxxtools::xmlrpc::RemoteProcedure<bool> multiply(client, "multiply");
+            cxxtools::xmlrpc::HttpClient client(_loop, "", _port + 1, "/rpc");
+            cxxtools::RemoteProcedure<bool> multiply(client, "multiply");
             connect( multiply.finished, *this, &XmlRpcTest::onConnectErrorCallback );
 
             multiply.begin();
 
             try
             {
-                _loop->run();
+                _loop.run();
             }
             catch (const std::exception& e)
             {
@@ -279,10 +216,10 @@ class XmlRpcTest : public cxxtools::unit::TestSuite
             }
         }
 
-        void onConnectErrorCallback(const cxxtools::xmlrpc::Result<bool>& r)
+        void onConnectErrorCallback(const cxxtools::RemoteResult<bool>& r)
         {
             log_debug("onConnectErrorCallback");
-            _loop->exit();
+            _loop.exit();
             CXXTOOLS_UNIT_ASSERT_THROW(r.get(), std::exception);
         }
 
@@ -293,28 +230,28 @@ class XmlRpcTest : public cxxtools::unit::TestSuite
         {
             cxxtools::xmlrpc::Service service;
             service.registerMethod("multiply", *this, &XmlRpcTest::multiplyBoolean);
-            _server->addService("/calc", service);
+            _server->addService("/rpc", service);
 
-            cxxtools::xmlrpc::HttpClient client(*_loop, "127.0.0.1", 8001, "/calc");
-            cxxtools::xmlrpc::RemoteProcedure<bool, bool, bool> multiply(client, "multiply");
+            cxxtools::xmlrpc::HttpClient client(_loop, "", _port, "/rpc");
+            cxxtools::RemoteProcedure<bool, bool, bool> multiply(client, "multiply");
             connect( multiply.finished, *this, &XmlRpcTest::onBooleanFinished );
 
             multiply.begin(true, true);
 
-            _loop->run();
+            _loop.run();
         }
 
-        void onBooleanFinished(const cxxtools::xmlrpc::Result<bool>& r)
+        void onBooleanFinished(const cxxtools::RemoteResult<bool>& r)
         {
-            CXXTOOLS_UNIT_ASSERT_EQUALS(r.get(), true)
+            CXXTOOLS_UNIT_ASSERT_EQUALS(r.get(), true);
 
-            _loop->exit();
+            _loop.exit();
         }
 
         bool multiplyBoolean(bool a, bool b)
         {
-            CXXTOOLS_UNIT_ASSERT_EQUALS(a, true)
-            CXXTOOLS_UNIT_ASSERT_EQUALS(b, true)
+            CXXTOOLS_UNIT_ASSERT_EQUALS(a, true);
+            CXXTOOLS_UNIT_ASSERT_EQUALS(b, true);
             return true;
         }
 
@@ -325,22 +262,22 @@ class XmlRpcTest : public cxxtools::unit::TestSuite
         {
             cxxtools::xmlrpc::Service service;
             service.registerMethod("multiply", *this, &XmlRpcTest::multiplyInt);
-            _server->addService("/calc", service);
+            _server->addService("/rpc", service);
 
-            cxxtools::xmlrpc::HttpClient client(*_loop, "127.0.0.1", 8001, "/calc");
-            cxxtools::xmlrpc::RemoteProcedure<int, int, int> multiply(client, "multiply");
+            cxxtools::xmlrpc::HttpClient client(_loop, "", _port, "/rpc");
+            cxxtools::RemoteProcedure<int, int, int> multiply(client, "multiply");
             connect( multiply.finished, *this, &XmlRpcTest::onIntegerFinished );
 
             multiply.begin(2, 3);
 
-            _loop->run();
+            _loop.run();
         }
 
-        void onIntegerFinished(const cxxtools::xmlrpc::Result<int>& r)
+        void onIntegerFinished(const cxxtools::RemoteResult<int>& r)
         {
-            CXXTOOLS_UNIT_ASSERT_EQUALS(r.get(), 6)
+            CXXTOOLS_UNIT_ASSERT_EQUALS(r.get(), 6);
 
-            _loop->exit();
+            _loop.exit();
         }
 
         int multiplyInt(int a, int b)
@@ -355,22 +292,22 @@ class XmlRpcTest : public cxxtools::unit::TestSuite
         {
             cxxtools::xmlrpc::Service service;
             service.registerMethod("multiply", *this, &XmlRpcTest::multiplyDouble);
-            _server->addService("/calc", service);
+            _server->addService("/rpc", service);
 
-            cxxtools::xmlrpc::HttpClient client(*_loop, "127.0.0.1", 8001, "/calc");
-            cxxtools::xmlrpc::RemoteProcedure<double, double, double> multiply(client, "multiply");
+            cxxtools::xmlrpc::HttpClient client(_loop, "", _port, "/rpc");
+            cxxtools::RemoteProcedure<double, double, double> multiply(client, "multiply");
             connect( multiply.finished, *this, &XmlRpcTest::onDoubleFinished );
 
             multiply.begin(2.0, 3.0);
 
-            _loop->run();
+            _loop.run();
         }
 
-        void onDoubleFinished(const cxxtools::xmlrpc::Result<double>& r)
+        void onDoubleFinished(const cxxtools::RemoteResult<double>& r)
         {
-            CXXTOOLS_UNIT_ASSERT_EQUALS(r.get(), 6.0)
+            CXXTOOLS_UNIT_ASSERT_EQUALS(r.get(), 6.0);
 
-            _loop->exit();
+            _loop.exit();
         }
 
         double multiplyDouble(double a, double b)
@@ -387,20 +324,20 @@ class XmlRpcTest : public cxxtools::unit::TestSuite
             service.registerMethod("echoString", *this, &XmlRpcTest::echoString);
             _server->addService("/foo", service);
 
-            cxxtools::xmlrpc::HttpClient client(*_loop, "127.0.0.1", 8001, "/foo");
-            cxxtools::xmlrpc::RemoteProcedure<std::string, std::string> echo(client, "echoString");
+            cxxtools::xmlrpc::HttpClient client(_loop, "", _port, "/foo");
+            cxxtools::RemoteProcedure<std::string, std::string> echo(client, "echoString");
             connect( echo.finished, *this, &XmlRpcTest::onStringEchoFinished );
 
             echo.begin("\xc3\xaf\xc2\xbb\xc2\xbf'\"&<> foo?");
 
-            _loop->run();
+            _loop.run();
         }
 
-        void onStringEchoFinished(const cxxtools::xmlrpc::Result<std::string>& r)
+        void onStringEchoFinished(const cxxtools::RemoteResult<std::string>& r)
         {
-            CXXTOOLS_UNIT_ASSERT_EQUALS(r.get(), "\xc3\xaf\xc2\xbb\xc2\xbf'\"&<> foo?")
+            CXXTOOLS_UNIT_ASSERT_EQUALS(r.get(), "\xc3\xaf\xc2\xbb\xc2\xbf'\"&<> foo?");
 
-            _loop->exit();
+            _loop.exit();
         }
 
         std::string echoString(std::string a)
@@ -415,27 +352,27 @@ class XmlRpcTest : public cxxtools::unit::TestSuite
         {
             cxxtools::xmlrpc::Service service;
             service.registerMethod("multiply", *this, &XmlRpcTest::multiplyEmpty);
-            _server->addService("/calc", service);
+            _server->addService("/rpc", service);
 
-            cxxtools::xmlrpc::HttpClient client(*_loop, "127.0.0.1", 8001, "/calc");
-            cxxtools::xmlrpc::RemoteProcedure<std::string, std::string, std::string> multiply(client, "multiply");
+            cxxtools::xmlrpc::HttpClient client(_loop, "", _port, "/rpc");
+            cxxtools::RemoteProcedure<std::string, std::string, std::string> multiply(client, "multiply");
             connect( multiply.finished, *this, &XmlRpcTest::onEmptyFinished );
 
             multiply.begin("", "");
 
-            _loop->run();
+            _loop.run();
         }
 
-        void onEmptyFinished(const cxxtools::xmlrpc::Result<std::string>& r)
+        void onEmptyFinished(const cxxtools::RemoteResult<std::string>& r)
         {
-            CXXTOOLS_UNIT_ASSERT_EQUALS(r.get(), "4")
-            _loop->exit();
+            CXXTOOLS_UNIT_ASSERT_EQUALS(r.get(), "4");
+            _loop.exit();
         }
 
         std::string multiplyEmpty(std::string a, std::string b)
         {
-            CXXTOOLS_UNIT_ASSERT_EQUALS(a, "")
-            CXXTOOLS_UNIT_ASSERT_EQUALS(b, "")
+            CXXTOOLS_UNIT_ASSERT_EQUALS(a, "");
+            CXXTOOLS_UNIT_ASSERT_EQUALS(b, "");
             return "4";
         }
 
@@ -446,10 +383,10 @@ class XmlRpcTest : public cxxtools::unit::TestSuite
         {
             cxxtools::xmlrpc::Service service;
             service.registerMethod("multiply", *this, &XmlRpcTest::multiplyVector);
-            _server->addService("/calc", service);
+            _server->addService("/rpc", service);
 
-            cxxtools::xmlrpc::HttpClient client(*_loop, "127.0.0.1", 8001, "/calc");
-            cxxtools::xmlrpc::RemoteProcedure< std::vector<int>, std::vector<int>, std::vector<int> > multiply(client, "multiply");
+            cxxtools::xmlrpc::HttpClient client(_loop, "", _port, "/rpc");
+            cxxtools::RemoteProcedure< std::vector<int>, std::vector<int>, std::vector<int> > multiply(client, "multiply");
             connect( multiply.finished, *this, &XmlRpcTest::onArrayFinished );
 
             std::vector<int> vec;
@@ -458,7 +395,7 @@ class XmlRpcTest : public cxxtools::unit::TestSuite
 
             multiply.begin(vec, vec);
 
-            _loop->run();
+            _loop.run();
         }
 
         std::vector<int> multiplyVector(const std::vector<int>& a, const std::vector<int>& b)
@@ -473,13 +410,13 @@ class XmlRpcTest : public cxxtools::unit::TestSuite
             return r;
         }
 
-        void onArrayFinished(const cxxtools::xmlrpc::Result<std::vector<int> >& r)
+        void onArrayFinished(const cxxtools::RemoteResult<std::vector<int> >& r)
         {
-            CXXTOOLS_UNIT_ASSERT_EQUALS(r.get().size(), 2)
-            CXXTOOLS_UNIT_ASSERT_EQUALS(r.get().at(0), 100)
-            CXXTOOLS_UNIT_ASSERT_EQUALS(r.get().at(1), 400)
+            CXXTOOLS_UNIT_ASSERT_EQUALS(r.get().size(), 2);
+            CXXTOOLS_UNIT_ASSERT_EQUALS(r.get().at(0), 100);
+            CXXTOOLS_UNIT_ASSERT_EQUALS(r.get().at(1), 400);
 
-            _loop->exit();
+            _loop.exit();
         }
 
         ////////////////////////////////////////////////////////////
@@ -489,23 +426,23 @@ class XmlRpcTest : public cxxtools::unit::TestSuite
         {
             cxxtools::xmlrpc::Service service;
             service.registerMethod("multiply", *this, &XmlRpcTest::multiplyVector);
-            _server->addService("/calc", service);
+            _server->addService("/rpc", service);
 
-            cxxtools::xmlrpc::HttpClient client(*_loop, "127.0.0.1", 8001, "/calc");
-            cxxtools::xmlrpc::RemoteProcedure< std::vector<int>, std::vector<int>, std::vector<int> > multiply(client, "multiply");
+            cxxtools::xmlrpc::HttpClient client(_loop, "", _port, "/rpc");
+            cxxtools::RemoteProcedure< std::vector<int>, std::vector<int>, std::vector<int> > multiply(client, "multiply");
             connect( multiply.finished, *this, &XmlRpcTest::onEmptyArrayFinished );
 
             std::vector<int> vec;
             multiply.begin(vec, vec);
 
-            _loop->run();
+            _loop.run();
         }
 
-        void onEmptyArrayFinished(const cxxtools::xmlrpc::Result<std::vector<int> >& r)
+        void onEmptyArrayFinished(const cxxtools::RemoteResult<std::vector<int> >& r)
         {
-            CXXTOOLS_UNIT_ASSERT_EQUALS(r.get().size(), 0)
+            CXXTOOLS_UNIT_ASSERT_EQUALS(r.get().size(), 0);
 
-            _loop->exit();
+            _loop.exit();
         }
 
         ////////////////////////////////////////////////////////////
@@ -515,10 +452,10 @@ class XmlRpcTest : public cxxtools::unit::TestSuite
         {
             cxxtools::xmlrpc::Service service;
             service.registerMethod("multiply", *this, &XmlRpcTest::multiplyColor);
-            _server->addService("/calc", service);
+            _server->addService("/rpc", service);
 
-            cxxtools::xmlrpc::HttpClient client(*_loop, "127.0.0.1", 8001, "/calc");
-            cxxtools::xmlrpc::RemoteProcedure< Color, Color, Color > multiply(client, "multiply");
+            cxxtools::xmlrpc::HttpClient client(_loop, "", _port, "/rpc");
+            cxxtools::RemoteProcedure< Color, Color, Color > multiply(client, "multiply");
             connect( multiply.finished, *this, &XmlRpcTest::onStuctFinished );
 
             Color a;
@@ -533,16 +470,16 @@ class XmlRpcTest : public cxxtools::unit::TestSuite
 
             multiply.begin(a, b);
 
-            _loop->run();
+            _loop.run();
         }
 
-        void onStuctFinished(const cxxtools::xmlrpc::Result<Color>& color)
+        void onStuctFinished(const cxxtools::RemoteResult<Color>& color)
         {
-            CXXTOOLS_UNIT_ASSERT_EQUALS(color.get().red, 6)
-            CXXTOOLS_UNIT_ASSERT_EQUALS(color.get().green, 12)
-            CXXTOOLS_UNIT_ASSERT_EQUALS(color.get().blue, 20)
+            CXXTOOLS_UNIT_ASSERT_EQUALS(color.get().red, 6);
+            CXXTOOLS_UNIT_ASSERT_EQUALS(color.get().green, 12);
+            CXXTOOLS_UNIT_ASSERT_EQUALS(color.get().blue, 20);
 
-            _loop->exit();
+            _loop.exit();
         }
 
         Color multiplyColor(const Color& a, const Color& b)
@@ -563,8 +500,8 @@ class XmlRpcTest : public cxxtools::unit::TestSuite
             service.registerMethod("multiplyset", *this, &XmlRpcTest::multiplySet);
             _server->addService("/test", service);
 
-            cxxtools::xmlrpc::HttpClient client(*_loop, "127.0.0.1", 8001, "/test");
-            cxxtools::xmlrpc::RemoteProcedure<IntSet, IntSet, int> multiply(client, "multiplyset");
+            cxxtools::xmlrpc::HttpClient client(_loop, "", _port, "/test");
+            cxxtools::RemoteProcedure<IntSet, IntSet, int> multiply(client, "multiplyset");
             connect( multiply.finished, *this, &XmlRpcTest::onSetFinished );
 
             IntSet myset;
@@ -575,10 +512,10 @@ class XmlRpcTest : public cxxtools::unit::TestSuite
 
             multiply.begin(myset, 2);
 
-            _loop->run();
+            _loop.run();
         }
 
-        void onSetFinished(const cxxtools::xmlrpc::Result<IntSet>& result)
+        void onSetFinished(const cxxtools::RemoteResult<IntSet>& result)
         {
             const IntSet& v = result.get();
             CXXTOOLS_UNIT_ASSERT_EQUALS(v.size(), 3);
@@ -586,7 +523,7 @@ class XmlRpcTest : public cxxtools::unit::TestSuite
             CXXTOOLS_UNIT_ASSERT(v.find(10) != v.end());
             CXXTOOLS_UNIT_ASSERT(v.find(22) != v.end());
 
-            _loop->exit();
+            _loop.exit();
         }
 
         IntSet multiplySet(const IntSet& s, int f)
@@ -606,8 +543,8 @@ class XmlRpcTest : public cxxtools::unit::TestSuite
             service.registerMethod("multiplyset", *this, &XmlRpcTest::multiplyMultiset);
             _server->addService("/test", service);
 
-            cxxtools::xmlrpc::HttpClient client(*_loop, "127.0.0.1", 8001, "/test");
-            cxxtools::xmlrpc::RemoteProcedure<IntMultiset, IntMultiset, int> multiply(client, "multiplyset");
+            cxxtools::xmlrpc::HttpClient client(_loop, "", _port, "/test");
+            cxxtools::RemoteProcedure<IntMultiset, IntMultiset, int> multiply(client, "multiplyset");
             connect( multiply.finished, *this, &XmlRpcTest::onMultisetFinished );
 
             IntMultiset myset;
@@ -618,10 +555,10 @@ class XmlRpcTest : public cxxtools::unit::TestSuite
 
             multiply.begin(myset, 2);
 
-            _loop->run();
+            _loop.run();
         }
 
-        void onMultisetFinished(const cxxtools::xmlrpc::Result<IntMultiset>& result)
+        void onMultisetFinished(const cxxtools::RemoteResult<IntMultiset>& result)
         {
             const IntMultiset& v = result.get();
             CXXTOOLS_UNIT_ASSERT_EQUALS(v.size(), 4);
@@ -629,7 +566,7 @@ class XmlRpcTest : public cxxtools::unit::TestSuite
             CXXTOOLS_UNIT_ASSERT_EQUALS(v.count(10), 2);
             CXXTOOLS_UNIT_ASSERT_EQUALS(v.count(22), 1);
 
-            _loop->exit();
+            _loop.exit();
         }
 
         IntMultiset multiplyMultiset(const IntMultiset& s, int f)
@@ -650,8 +587,8 @@ class XmlRpcTest : public cxxtools::unit::TestSuite
 
             _server->addService("/test", service);
 
-            cxxtools::xmlrpc::HttpClient client(*_loop, "127.0.0.1", 8001, "/test");
-            cxxtools::xmlrpc::RemoteProcedure<IntMap, IntMap, int> multiply(client, "multiplymap");
+            cxxtools::xmlrpc::HttpClient client(_loop, "", _port, "/test");
+            cxxtools::RemoteProcedure<IntMap, IntMap, int> multiply(client, "multiplymap");
             connect( multiply.finished, *this, &XmlRpcTest::onMultiplyMapFinished );
 
             IntMap mymap;
@@ -661,10 +598,10 @@ class XmlRpcTest : public cxxtools::unit::TestSuite
 
             multiply.begin(mymap, 2);
 
-            _loop->run();
+            _loop.run();
         }
 
-        void onMultiplyMapFinished(const cxxtools::xmlrpc::Result<IntMap>& result)
+        void onMultiplyMapFinished(const cxxtools::RemoteResult<IntMap>& result)
         {
             const IntMap& v = result.get();
             CXXTOOLS_UNIT_ASSERT_EQUALS(v.size(), 3);
@@ -675,7 +612,7 @@ class XmlRpcTest : public cxxtools::unit::TestSuite
             CXXTOOLS_UNIT_ASSERT(v.find(1) != v.end());
             CXXTOOLS_UNIT_ASSERT_EQUALS(v.find(1)->second, -2);
 
-            _loop->exit();
+            _loop.exit();
         }
 
         IntMap multiplyMap(const IntMap& m, int f)
@@ -699,8 +636,8 @@ class XmlRpcTest : public cxxtools::unit::TestSuite
 
             _server->addService("/test", service);
 
-            cxxtools::xmlrpc::HttpClient client(*_loop, "127.0.0.1", 8001, "/test");
-            cxxtools::xmlrpc::RemoteProcedure<IntMultimap, IntMultimap, int> multiply(client, "multiplymultimap");
+            cxxtools::xmlrpc::HttpClient client(_loop, "", _port, "/test");
+            cxxtools::RemoteProcedure<IntMultimap, IntMultimap, int> multiply(client, "multiplymultimap");
             connect( multiply.finished, *this, &XmlRpcTest::onMultiplyMultimapFinished );
 
             IntMultimap mymap;
@@ -711,10 +648,10 @@ class XmlRpcTest : public cxxtools::unit::TestSuite
 
             multiply.begin(mymap, 2);
 
-            _loop->run();
+            _loop.run();
         }
 
-        void onMultiplyMultimapFinished(const cxxtools::xmlrpc::Result<IntMultimap>& result)
+        void onMultiplyMultimapFinished(const cxxtools::RemoteResult<IntMultimap>& result)
         {
             const IntMultimap& v = result.get();
             CXXTOOLS_UNIT_ASSERT_EQUALS(v.size(), 4);
@@ -730,7 +667,7 @@ class XmlRpcTest : public cxxtools::unit::TestSuite
             CXXTOOLS_UNIT_ASSERT(v.lower_bound(1) != v.end());
             CXXTOOLS_UNIT_ASSERT_EQUALS(v.lower_bound(1)->second, -2);
 
-            _loop->exit();
+            _loop.exit();
         }
 
         IntMultimap multiplyMultimap(const IntMultimap& m, int f)
@@ -744,6 +681,137 @@ class XmlRpcTest : public cxxtools::unit::TestSuite
             return ret;
         }
 
+        void UnknownMethod()
+        {
+            cxxtools::xmlrpc::HttpClient client(_loop, "", _port, "/test");
+            cxxtools::RemoteProcedure<bool, bool, bool> unknownMethod(client, "unknownMethod");
+            connect( unknownMethod.finished, *this, &XmlRpcTest::onBooleanFinished );
+
+            unknownMethod.begin(true, true);
+
+            CXXTOOLS_UNIT_ASSERT_THROW(_loop.run(), std::exception);
+        }
+
+        ////////////////////////////////////////////////////////////
+        // Fault
+        //
+        void Fault()
+        {
+            cxxtools::xmlrpc::Service service;
+            service.registerMethod("multiply", *this, &XmlRpcTest::throwFault);
+            _server->addService("/rpc", service);
+
+            cxxtools::xmlrpc::HttpClient client(_loop, "", _port, "/rpc");
+            cxxtools::RemoteProcedure<bool> multiply(client, "multiply");
+            connect( multiply.finished, *this, &XmlRpcTest::onFault );
+            multiply.begin();
+
+            _loop.run();
+        }
+
+        void onFault(const cxxtools::RemoteResult<bool>& result)
+        {
+            try
+            {
+                result.get();
+                CXXTOOLS_UNIT_ASSERT_MSG(false, "cxxtools::RemoteException exception expected");
+            }
+            catch (const cxxtools::RemoteException& e)
+            {
+                CXXTOOLS_UNIT_ASSERT_EQUALS(e.rc(), 7);
+                CXXTOOLS_UNIT_ASSERT_EQUALS(e.text(), "Fault");
+            }
+
+            _loop.exit();
+        }
+
+        bool throwFault()
+        {
+            throw cxxtools::RemoteException("Fault", 7);
+            return false;
+        }
+
+        ////////////////////////////////////////////////////////////
+        // Exception
+        //
+        void Exception()
+        {
+            cxxtools::xmlrpc::Service service;
+            service.registerMethod("multiply", *this, &XmlRpcTest::throwException);
+            _server->addService("/rpc", service);
+
+            cxxtools::xmlrpc::HttpClient client(_loop, "", _port, "/rpc");
+            cxxtools::RemoteProcedure<bool> multiply(client, "multiply");
+            connect( multiply.finished, *this, &XmlRpcTest::onException );
+            multiply.begin();
+
+            _loop.run();
+        }
+
+        void onException(const cxxtools::RemoteResult<bool>& result)
+        {
+            try
+            {
+                result.get();
+                CXXTOOLS_UNIT_ASSERT(false);
+            }
+            catch (const cxxtools::RemoteException& e)
+            {
+                CXXTOOLS_UNIT_ASSERT_EQUALS(e.rc(), 0);
+                CXXTOOLS_UNIT_ASSERT_EQUALS(e.text(), "Exception");
+            }
+
+            _loop.exit();
+        }
+
+        bool throwException()
+        {
+            throw std::runtime_error("Exception");
+            return false;
+        }
+
+        ////////////////////////////////////////////////////////////
+        // BigRequest
+        //
+        void BigRequest()
+        {
+            log_trace("ConnectError");
+
+            cxxtools::xmlrpc::Service service;
+            service.registerMethod("countSize", *this, &XmlRpcTest::countSize);
+            _server->addService("/rpc", service);
+
+            cxxtools::xmlrpc::HttpClient client(_loop, "", _port, "/rpc");
+            cxxtools::RemoteProcedure<unsigned, std::vector<int> > countSize(client, "countSize");
+            connect( countSize.finished, *this, &XmlRpcTest::onCountSizeFinished );
+
+            std::vector<int> v;
+            v.resize(5000);
+
+            countSize.begin(v);
+
+            try
+            {
+                _loop.run();
+            }
+            catch (const std::exception& e)
+            {
+                log_error("loop exited with exception: " << e.what());
+                CXXTOOLS_UNIT_ASSERT_MSG(false, std::string("unexpected exception ") + typeid(e).name() + ": " + e.what());
+            }
+        }
+
+        void onCountSizeFinished(const cxxtools::RemoteResult<unsigned>& r)
+        {
+            CXXTOOLS_UNIT_ASSERT_EQUALS(r.get(), 5000);
+            _loop.exit();
+        }
+
+        unsigned countSize(const std::vector<int>& v)
+        {
+            return v.size();
+        }
+
 };
 
 cxxtools::unit::RegisterTest<XmlRpcTest> register_XmlRpcTest;
